"""
Created on Fri Sep  6 14:53:33 2019

@author: RdelRioChanona
This script plots the centrality measures of a given files for a listed group
of layers
"""
import numpy as np
import pandas as pd
from matplotlib import pyplot as plt
import matplotlib.gridspec as gridspec

## First define some local folder path
data_path = './data/'
csv_files_path = './data/csv_files'
export_path_csv = './results/real_results_csv/'
export_path_fig = './results/fig/'

file_name_mx = "mx_layercentrality_nocar_3lay"
car_name = ""
#name = "Katz"
name = "MultiRank PageRank"
name = "Money percent"
export_fig_name = name+"_3lay.svg"
layer_list = ["Equity", "Debt", "Bank"]

y_start  = 2008
y_end = 2019

def layer_score(df_name, centrality_name, layer_list, y_start=2008,\
    y_end=2019):
    """
    function that returns the rank of each country in list according to the
    centrality measure
    """
    score = {}
    for lay in layer_list:
        score[lay] = []
    for y in range(y_start,y_end):
        df = pd.read_csv(export_path_csv + df_name + str(y) + ".csv")

        for layer in layer_list:
            #add rank to dict, +1 to avoid 0-th rank
            score[layer].append(df[centrality_name + " score"][df[centrality_name] == layer].values[0])
    return score

score_katz_mx = layer_score(file_name_mx, "Katz", layer_list, y_start=y_start)
score_PageRank_mx = layer_score(file_name_mx, "MultiRank PageRank", layer_list,y_start=y_start)
score_MoneyPercent_mx = layer_score(file_name_mx, "Money percent", layer_list, y_start=y_start)

score_mx = layer_score(file_name_mx, name, layer_list, y_start=y_start)

color = ["#a65628", "#4daf4a", "#377eb8" ]
marker_style = ["--",":", "-.", "-", "-"]
fontsize_ticks = 30
fontsize_axis = 30
fontsize_title = 40
fontsize_legend = 30

f = plt.figure(figsize=(20,10))
# Uncomment if removing labels
#f = plt.figure(figsize=(10,10))
f.subplots_adjust(hspace=0.2)
gs = gridspec.GridSpec(1, 1)
ax = plt.subplot(gs[0])
ax.spines['right'].set_visible(False)
ax.spines['top'].set_visible(False)
ax.tick_params(labelsize=fontsize_ticks)

for l, layer in enumerate(layer_list):
    ax.plot([i for i in range(y_start,y_end)], score_mx[layer], \
             label=["Equity", "Debt", "Banking"][l], linewidth=5.0, linestyle=marker_style[l],\
             color=color[l])
# Shrink current axis by 20%
box = ax.get_position()
ax.set_position([box.x0, box.y0, box.width * 0.8, box.height])
#     Put a legend to the right of the current axis
ax.legend(loc='center left', bbox_to_anchor=(1, 0.5), fontsize=fontsize_legend)
# ax.set_ylabel(name, fontsize=fontsize_axis)
ax.set_ylabel("Fraction of assets", fontsize=fontsize_axis)
#ax.set_ylabel("Layer influence", fontsize=fontsize_axis)
ax.set_xlabel("Year", fontsize=fontsize_axis)
# ax1.legend()
ax.set_xticks([y_start + 2*i for i in range(6)])
ax.set_ylim([0,0.8])
plt.title("", fontsize=fontsize_title)
plt.savefig(export_path_fig + car_name + export_fig_name, bbox_inches="tight")
plt.show()
