# -*- coding: utf-8 -*-
"""
Created on Fri Sep  6 14:53:33 2019

@author: RdelRioChanona
This script plot the centrality measures of a given files for a listed group
of countries
Does for equity, debt, bank, aggregate and multiplex network
"""
import numpy as np
import pandas as pd
from matplotlib import pyplot as plt
import matplotlib.gridspec as gridspec

## First define some local folder path
data_path = './data/'
csv_files_path = './data/csv_files'
export_path_csv = './results/real_results_csv/'
export_path_fig = './results/fig/'

file_name_mx = "mx_centrality_3lay"
file_name_agg = "agg_centrality_3lay"
file_name_mono = "mono_centrality_3lay"
file_name_agg = file_name_mono
car_name = ""

year = 2018
df_mx = pd.read_csv(export_path_csv + file_name_mx + str(year) + ".csv")
df_agg = pd.read_csv(export_path_csv + file_name_agg + str(year) + ".csv")
df_mono = pd.read_csv(export_path_csv + file_name_mono + str(year) + ".csv")
df_agg = df_mono


def systemic_countries(df_name, centrality_name,  y_start=2008,\
    y_end=2019):
    """
    function that returns the rank of each country in list according to the
    centrality measure
    """
    systemic = {}
    country_list = []
    # make list of all countries that are systemic at some point
    for y in range(y_start,y_end):
        df = pd.read_csv(export_path_csv + df_name + str(y) + ".csv")
        country_list = country_list + list(df[centrality_name]\
                    [df[centrality_name + " score"]>1])

    for country in country_list:
        systemic[country] = []
        for y in range(y_start,y_end):
            # 0 for not systemic, 1 for systemic
#            print(y)
#            print(df[centrality_name + " score"][df[centrality_name]==country].values[0])
#            print(df[centrality_name + " score"][df[centrality_name]==country])
            df = pd.read_csv(export_path_csv + df_name + str(y) + ".csv")
            if df[centrality_name + " score"][df[centrality_name]==country].values[0] > 1:
                systemic[country].append(1)
            else:
                systemic[country].append(0)

    return systemic

marker_style = ["--",":",  "-", "-.", "-"]

systemic_mx_33nocar = systemic_countries(file_name_mx, \
                                         "Cascade with tie break 33")

systemic_mx_09010car  = systemic_countries(file_name_mx, \
                                          "Cascade with tie break 33 0_09car")
systemic_mx_08010car  = systemic_countries(file_name_mx, \
                                          "Cascade with tie break 33 0_08car")
systemic_mx_06710car  = systemic_countries(file_name_mx, \
                                          "Cascade with tie break 33 0_67car")

systemic_mx_3310car  = systemic_countries(file_name_mx, \
                                          "Cascade with tie break 33 10car")
systemic_mx_3320car  = systemic_countries(file_name_mx, \
                                          "Cascade with tie break 33 20car")
systemic_mx_3333car  = systemic_countries(file_name_mx, \
                                              "Cascade with tie break 33 33car")
fontsize_ticks = 30
fontsize_axis = 30
fontsize_title = 40
fontsize_legend = 30

def plot_centrality(rank_dict, title_name, save_name, line_color="b"):
    f = plt.figure(figsize=(10,10))
    f.subplots_adjust(hspace=0.2)
    gs = gridspec.GridSpec(1, 1)
    ax = plt.subplot(gs[0])
    ax.spines['right'].set_visible(False)
    ax.spines['top'].set_visible(False)
    ax.tick_params(labelsize=fontsize_ticks)

    for j, country in enumerate(rank_dict):
        ax.plot([i for i in range(2008,2019)],rank_dict[country], \
                 label=country, linewidth=5.0, linestyle=marker_style[j],\
                 color=line_color)
    # Shrink current axis by 20%
    box = ax.get_position()
    ax.set_position([box.x0, box.y0, box.width * 0.8, box.height])
    # Put a legend to the right of the current axis
    ax.legend(loc='center left', bbox_to_anchor=(1, 0.5), \
              fontsize=fontsize_legend)
    ax.set_ylabel("Rank", fontsize=fontsize_axis)
    ax.set_xlabel("Year", fontsize=fontsize_axis)
    # ax1.legend()
    ax.set_ylim([0, 25])
    plt.gca().invert_yaxis()
    plt.title(title_name, fontsize=fontsize_title)
    plt.ylabel("Rank")
    plt.xlabel("year")
    plt.savefig(save_name, bbox_inches="tight")
    plt.show()

color = ["#601A4A", "#EE442F", "#63ACBE", "b" ]

group_top = ["United States", "United Kingdom"]
group_europe = ["Italy", "Spain", "Germany",  "France"]
group_asia = ["Hong Kong SAR", "China", "Singapore"]
all_groups = [group_top, group_europe, group_asia]
### all in one
color = ["#601A4A", "#EE442F", "#63ACBE" ]

#### aggregate

group_top = ["United States", "United Kingdom"]
group_europe = ["Italy", "Spain", "Germany",  "France"]
group_asia = ["Hong Kong SAR", "China", "Singapore"]
all_groups = [group_top, group_europe, group_asia]
all_countries = group_top + group_europe + group_asia
### all in one
color = ["#601A4A", "#EE442F", "#63ACBE", "b"]

centrality_list_agg = ["Cascade with tie break equity", "Cascade with tie break debt",\
                       "Cascade with tie break bank", "Cascade with tie break agg"]
# edit to plot MultiRank here
centrality_list_mx = ["Cascade with tie break 33", "Cascade with tie break 50",\
                       "Cascade with tie break 66", "Cascade with tie break 80"]

centrality_list_mx_horizontalvar = ['Cascade with tie break 33 10car',\
                       'Cascade with tie break 33 20car','Cascade with tie break 33 33car',\
                       "Cascade with tie break 33",\
                       'Cascade with tie break 33 0_09car', 'Cascade with tie break 33 0_08car', \
                       'Cascade with tie break 33 0_67car']
centrality_list_agg_horizontalvar = [ 'Cascade with tie break 10car',\
                       'Cascade with tie break 20car', 'Cascade with tie break 33car',\
                       "Cascade with tie break agg",'Cascade with tie break 0_09car', \
                       'Cascade with tie break 0_08car', \
                       'Cascade with tie break 0_67car']

### Counting number of nodes that default

def count_defaulted(df_name, centrality_name, y_start=2008,\
    y_end=2019):
    """
    function that returns the rank of each country in list according to the
    centrality measure
    """
    defaulted_countries = []
    for y in range(y_start,y_end):
        df = pd.read_csv(export_path_csv + df_name + str(y) + ".csv")
        defaulted_countries.append(np.where(df[centrality_name + \
                                            " score"].values >= 1)[0].shape[0])

    return defaulted_countries



# =============================================================================
# Plot number of systemic vertical variation
# =============================================================================

color1 = ["#63ACBE", "#EE442F", "#601A4A", "k"]
names1 = ["Mx, c=0.33", "Mx, c=0.50", "Mx, c=0.66", "Mx, c=0.80"]

color2 = ["#a65628", "#4daf4a", "#377eb8", "k" ]
names2 = ["Equity", "Debt", "Banking", "Aggregate"]

f = plt.figure(figsize=(20,10))
# Uncomment if removing labels
#f = plt.figure(figsize=(10,10))
f.subplots_adjust(hspace=0.2)
gs = gridspec.GridSpec(1, 1)
ax = plt.subplot(gs[0])
ax.spines['right'].set_visible(False)
ax.spines['top'].set_visible(False)
ax.tick_params(labelsize=fontsize_ticks)
for i, centrality in enumerate(centrality_list_mx):

    defaults = count_defaulted(file_name_mx, centrality)
    ax.plot([i for i in range(2008,2019)], defaults, \
                 label=names1[i], linewidth=5.0, linestyle=marker_style[0],\
                 color=color1[i])
    print(centrality, names1[i])

for i, centrality in enumerate(centrality_list_agg):

    defaults = count_defaulted(file_name_agg, centrality)
    ax.plot([i for i in range(2008,2019)], defaults, \
                 label=names2[i], linewidth=5.0, linestyle=":", \
#                 linestyle=marker_style[2],\
                 color=color2[i])

## Shrink current axis by 20%
box = ax.get_position()
ax.set_position([box.x0, box.y0, box.width * 0.8, box.height])
#     Put a legend to the right of the current axis
ax.legend(loc='center left', bbox_to_anchor=(1, 0.5), fontsize=fontsize_legend)
ax.set_ylabel("Number of systemic countries", fontsize=fontsize_axis)
ax.set_xlabel("Year", fontsize=fontsize_axis)
plt.savefig(export_path_fig + car_name + "SystemicCountries_car.png", bbox_inches="tight")
plt.show()


color1 = [ "#63ACBE", "#EE442F", "#601A4A", "k", "#63ACBE", "#EE442F", "#601A4A",]
names1 = [ "$b_i=1.1\,\mathrm{CAR}_i$", "$b_i=1.2\,\mathrm{CAR}_i$", \
          "$b_i=1.3\,\mathrm{CAR}_i$","$b_i=\mathrm{CAR}_i$","$b_i=0.9\,\mathrm{CAR}_i$","$b_i=0.8\,\mathrm{CAR}_i$",\
          "$b_i=0.67\,\mathrm{CAR}_i$"]

marker_style = [ "--","--","--","-","-.","-.", "-."]

color2 = ["#a65628", "#4daf4a", "#377eb8", "k", "#a65628", "#4daf4a", "#377eb8", ]
names2 = [ "$b_i=1.1\,\mathrm{CAR}_i$", "$b_i=1.2\,\mathrm{CAR}_i$", \
          "$b_i=1.3\,\mathrm{CAR}_i$","$b_i=\,\mathrm{CAR}_i$","$b_i=0.9\,\mathrm{CAR}_i$","$b_i=0.8\,\mathrm{CAR}_i$",\
          "$b_i=0.67\,\mathrm{CAR}_i$"]


# =============================================================================
# Plot Number of systemic horizontal threshold
# =============================================================================


f = plt.figure(figsize=(20,10))
# Uncomment if removing labels
#f = plt.figure(figsize=(10,10))
f.subplots_adjust(hspace=0.2)
gs = gridspec.GridSpec(1, 1)
ax = plt.subplot(gs[0])
ax.spines['right'].set_visible(False)
ax.spines['top'].set_visible(False)
ax.tick_params(labelsize=fontsize_ticks)
for i, centrality in enumerate(centrality_list_agg_horizontalvar):

    defaults = count_defaulted(file_name_agg, centrality)
    ax.plot([i for i in range(2008,2019)], defaults, \
                 label=names2[i], linewidth=5.0, linestyle=marker_style[i],\
                 color=color1[i])

## Shrink current axis by 20%
handles, labels = ax.get_legend_handles_labels()
box = ax.get_position()
ax.set_position([box.x0, box.y0, box.width * 0.8, box.height])
#     Put a legend to the right of the current axis
ax.legend(handles[::-1], labels[::-1],loc='center left', bbox_to_anchor=(1, 0.5), fontsize=fontsize_legend)
ax.set_ylabel("Number of systemic countries", fontsize=fontsize_axis)
ax.set_xlabel("Year", fontsize=fontsize_axis)
ax.set_ylim([0, 32])
#ax.set_yticks([1, 5, 10, 15, 20, 25])
#ax.set_xticks([2008 + 2*i for i in range(6)])
#plt.title("Aggregate monoplex", fontsize=fontsize_title)
#plt.ylabel("Rank            ", rotation=00)
#plt.xlabel("Year")
plt.savefig(export_path_fig + car_name + "SystemicCountries_hor_agg2.png", bbox_inches="tight")
plt.savefig(export_path_fig + car_name + "SystemicCountries_hor_agg2.svg", bbox_inches="tight")
plt.show()

f = plt.figure(figsize=(20,10))
# Uncomment if removing labels
#f = plt.figure(figsize=(10,10))
f.subplots_adjust(hspace=0.2)
gs = gridspec.GridSpec(1, 1)
ax = plt.subplot(gs[0])
ax.spines['right'].set_visible(False)
ax.spines['top'].set_visible(False)
ax.tick_params(labelsize=fontsize_ticks)

for i, centrality in enumerate(centrality_list_mx_horizontalvar):

    defaults = count_defaulted(file_name_mx, centrality)
    ax.plot([i for i in range(2008,2019)], defaults, \
                 label=names1[i], linewidth=5.0, linestyle=marker_style[i],\
                 color=color1[i])

## Shrink current axis by 20%
handles, labels = ax.get_legend_handles_labels()
box = ax.get_position()
ax.set_position([box.x0, box.y0, box.width * 0.8, box.height])
#     Put a legend to the right of the current axis
ax.legend(handles[::-1], labels[::-1],loc='center left', bbox_to_anchor=(1, 0.5), fontsize=fontsize_legend)
ax.set_ylabel("Number of systemic countries", fontsize=fontsize_axis)
ax.set_xlabel("Year", fontsize=fontsize_axis)
#ax.set_ylim([0, 32])
#ax.set_yticks([1, 5, 10, 15, 20, 25])
#ax.set_xticks([2008 + 2*i for i in range(6)])
#plt.title("Multiplex", fontsize=fontsize_title)
#plt.ylabel("Rank            ", rotation=00)
#plt.xlabel("Year")
plt.savefig(export_path_fig + car_name + "SystemicCountries_hor_mx.png", bbox_inches="tight")
plt.savefig(export_path_fig + car_name + "SystemicCountries_hor_mx.svg", bbox_inches="tight")
plt.show()
