# -*- coding: utf-8 -*-
"""
Created on Wed May 27 15:34:08 2020

@author: rmari
Does plot, for countries that are systemic for more than 4 years of baseline 
scenario, check when they are systemic according to CAR.
"""
import numpy as np
import pandas as pd
from matplotlib import pyplot as plt
import matplotlib
## First define some local folder path
data_path = './data/'
csv_files_path = './data/csv_files'
export_path_csv = './results/real_results_csv/'
export_path_fig = './results/fig/'

file_name_mx = "mx_centrality_3lay"
file_name_agg = "agg_centrality_3lay"
file_name_mono = "mono_centrality_3lay"
file_name_agg = file_name_mono
car_name = ""

year = 2018
df_mx = pd.read_csv(export_path_csv + file_name_mx + str(year) + ".csv")
df_agg = pd.read_csv(export_path_csv + file_name_agg + str(year) + ".csv")
df_mono = pd.read_csv(export_path_csv + file_name_mono + str(year) + ".csv")
df_agg = df_mono


def systemic_countries(df_name, centrality_name,  y_start=2008,\
    y_end=2019):
    """
    function that returns the rank of each country in list according to the
    centrality measure
    """
    systemic = {}
    country_list = []
    # make list of all countries that are systemic at some point
    for y in range(y_start,y_end):
        df = pd.read_csv(export_path_csv + df_name + str(y) + ".csv")
        country_list = country_list + list(df[centrality_name]\
                    [df[centrality_name + " score"]>1])
    
    for country in country_list:
        systemic[country] = []
        for y in range(y_start,y_end):
            # 0 for not systemic, 1 for systemic
#            print(y)
#            print(df[centrality_name + " score"][df[centrality_name]==country].values[0])
#            print(df[centrality_name + " score"][df[centrality_name]==country])
            df = pd.read_csv(export_path_csv + df_name + str(y) + ".csv")
            if df[centrality_name + " score"][df[centrality_name]==country].values[0] > 1:
                systemic[country].append(1)
            else:
                systemic[country].append(0)
             
    return systemic

marker_style = ["--",":",  "-", "-.", "-"]

systemic_mx_33baselinecar = systemic_countries(file_name_mx, \
                                         "Cascade with tie break 33")

systemic_mx_09010car  = systemic_countries(file_name_mx, \
                                          "Cascade with tie break 33 0_09car")
systemic_mx_08010car  = systemic_countries(file_name_mx, \
                                          "Cascade with tie break 33 0_08car")
systemic_mx_06710car  = systemic_countries(file_name_mx, \
                                          "Cascade with tie break 33 0_67car")

systemic_mx_3310car  = systemic_countries(file_name_mx, \
                                          "Cascade with tie break 33 10car")
systemic_mx_3320car  = systemic_countries(file_name_mx, \
                                          "Cascade with tie break 33 20car")
systemic_mx_3333car  = systemic_countries(file_name_mx, \
                                              "Cascade with tie break 33 33car")

list_systemic = [systemic_mx_06710car, systemic_mx_08010car, \
                systemic_mx_09010car, systemic_mx_33baselinecar,\
                systemic_mx_3310car, systemic_mx_3320car, systemic_mx_3333car]

def select_countries(dict_country, n_times_systemic=4):
    """Given a dictionary of countrie and list of when country is systemic
    returns list of countries that are systemic in more than one year
    """
    list_countries = []
    for country in dict_country.keys():
        if sum(dict_country[country]) > 4:
            list_countries.append(country)
    return list_countries

selected_countries = select_countries(systemic_mx_33baselinecar)
# reshuffling
selected_countries = ['United States', 'United Kingdom', 'Luxembourg',\
                       'Netherlands','Germany','France','Italy','Switzerland',\
                       'Canada','Japan','China','Hong Kong SAR','Singapore']
n_selected = len(selected_countries)
n_years = 11

matrix = np.zeros([n_selected, int(n_years*7) ])

matrix= np.empty([n_selected, int(n_years*7) ])
matrix[:] = np.nan

ticks_x = [ 3 + i*len(list_systemic) for i in range(int(n_years))]
ticke_labels_x = [str(i) for i in range(2008,2019)]
# c index will be used to fill matrix, contry to access dictionary
for c, country in enumerate(selected_countries):
    # count to fill in row year and type cat
    count = 0
    # iterate year
    for year in range(n_years):
        # iterate type car
        for s, sys in enumerate(list_systemic):
            # if for given car the country in that year is systemic fill matrix
            try:
                if sys[country][year] == 1:
                    matrix[c, count] = s + 1 # +1 to make different than zero
                
            except:
                pass
            count += 1 # update counter
            
figsize_ = (26,16)
fontsize_ticks = 20
fontsize_axis = 26
fontsize_title = 40
fontsize_legend = 18
linewidth_ = 4

plt.figure(figsize=figsize_)
ax = plt.gca() 
img = plt.imshow(matrix, cmap="RdYlBu_r", interpolation='nearest', aspect="auto" )
ax.set_yticks( [-0.5] + [i +0.5 for i in range(len(selected_countries))])  
ax.grid(True, color='k', linestyle='--', linewidth=2, alpha=0.5, axis='y')

plt.show()


plt.figure(figsize=figsize_)
ax = plt.gca() 
img = plt.imshow(matrix, cmap="RdYlBu_r", interpolation='nearest', aspect="auto" )

current_cmap = matplotlib.cm.get_cmap()
current_cmap.set_bad(color='k')   
plt.xticks(ticks_x)  
plt.gca().set_xticklabels(ticke_labels_x, fontsize=fontsize_ticks)  
ax.set_yticks( [-0.5] + [i +0.5 for i in range(len(selected_countries))])  

ax.set_yticklabels(selected_countries + [""], fontsize=fontsize_ticks)  
ax.grid(True, color='k', linestyle='--', linewidth=2, alpha=0.5, axis='y')
cbar = plt.colorbar(img, orientation="horizontal", pad=0.05)
cbar.set_ticks([i for i in range(9)])
cbar.set_ticklabels(["$b_i=1.1\,\mathrm{CAR}_i$", "$b_i=0.67\,\mathrm{CAR}_i$", \
          "$b_i=0.8\,\mathrm{CAR}_i$","$b_i=0.9\,\mathrm{CAR}_i$","$b_i=\mathrm{CAR}_i$","$b_i=1.1\,\mathrm{CAR}_i$",\
          "$b_i=1.2\,\mathrm{CAR}_i$", "$b_i=1.33\,\mathrm{CAR}_i$",])
#setp(cbar.ax.yaxis.get_ticklabels(), fontsize=16)
plt.savefig(export_path_fig + "country_table.png", bbox_inches="tight")
plt.savefig(export_path_fig + "country_table.svg", bbox_inches="tight")
plt.show()

   