#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Created on Tue Aug 20 21:20:42 2019

@author: ritamaria
This script ranks countries according to different centality measures,
 Ranking done for the multiplex and aggregate networks
 Only considers countries that have CAR
"""
import sys
import numpy as np
import pandas as pd
lib_path = './lib'
sys.path.insert(0, lib_path)
import sprmulticontagion as mc

year = 2018
## First define some local folder path
data_path = './data/'
export_path_csv = './results/csv_internship/'
export_path_fig = './results/fig_internship/'
multiplex_file = "multiplex_caronly_edgelist_NoBankingLayer.csv"
car_file = "CAR.xlsx"
# save csv file name
no_car = False
if no_car:
    exp_file_name_mx = "mx_centrality_nocar_3lay"
    exp_file_name_mxlayer = "mx_layercentrality_nocar_3lay"
    exp_file_name_agg = "agg_centrality_nocar_3lay"
    exp_file_name_mono = "mono_centrality_nocar_3lay"
else:
    exp_file_name_mx = "mx_centrality_3lay"
    exp_file_name_mxlayer = "mx_layercentrality_3lay"
    exp_file_name_agg = "agg_centrality_3lay"
    exp_file_name_mono = "mono_centrality_3lay"


## Threshold data
# df_car_mean = pd.read_csv(data_path + car_file )
# Take the average CAR for each country
df_car =  pd.read_excel(data_path + car_file, sheet_name='Sheet3' )
df_car_last5 = df_car[["Country","2013","2014","2015","2016","2017","2018"]]
df_car_mean = df_car_last5
df_car_mean["mean_car"] = df_car_last5.mean(axis=1)
df_car_mean = df_car_mean[np.isfinite(df_car_mean['mean_car'])]

# get the countries that have car and make dictionary
countries_in_car = set(df_car_mean["Country"].value_counts().index.tolist())
dict_car = dict(zip(df_car_mean.Country, df_car_mean.mean_car))

### Network data
# multiplex network
df = pd.read_csv(data_path + multiplex_file)
df['Equity'] = df.CDIS_IADE + df.CPIS_IAPE
df["Debt"] = df.CDIS_IADD + df.CPIS_IAPD
df["Bank"] = df.loans_dep
df_multiplex = df[["country","counterpart", "Equity", "Debt", \
                          "Bank"]][(df["year"] ==year)]


# aggregate network
df_aggregate = df_multiplex[["country","counterpart"]]
df_aggregate = df_aggregate.assign(All_weights=\
                                   df_multiplex.sum(axis=1).values)

df_equity = df_multiplex[["country","counterpart", "Equity"]]
df_debt = df_multiplex[["country","counterpart", "Debt"]]
df_bank = df_multiplex[["country","counterpart", "Bank"]]
### Making networks
mx_net = mc.MultiplexNet(df_multiplex)
agg_net = mc.MultiplexNet(df_aggregate)
equity_net = mc.MultiplexNet(df_equity)
debt_net = mc.MultiplexNet(df_debt)
bank_net = mc.MultiplexNet(df_bank)

# defining thresholds and equity vectors
car = np.array([dict_car[mx_net.nodes[i]] for i in \
                       range(mx_net.nnodes)])
thresholds = 0.01*car  # from percentage points to ratio
homogenous_threshold = thresholds.mean()*np.ones(agg_net.nnodes)
#if no_car:
#    #thresholds = homogenous_threshold
#    thresholds = np.ones(mx_net.nnodes)
#    car =  np.ones(mx_net.nnodes)

# equity is total assets * capital to asset ratio
buffer_agg = np.multiply(agg_net.adj_matrix_list[0].sum(axis=1), 0.01*car)
buffer_mx = [np.multiply(mx_net.adj_matrix_list[l].sum(axis=1), 0.01*car) \
             for l in range(mx_net.nlayers)]

#### Ranking networks multialyer
pagerank_mx = mx_net.pagerank()
hubs_mx, auth_mx = mx_net.hubs_and_auth()
multirank_mx, multirank_layers = mx_net.multirank()
#multirank_mx_unw, multirank_layers_unw = mx_net.multirank(a=0)
multirank_katz, multirank_katz_layers = mx_net.multirank(mode="katz")
#multirank_katz_unw, multirank_katz_layers_unw = mx_net.multirank(mode="katz", \
#    v_katz=thresholds, a=0)
money_percent = mx_net.layerrank_moneypercent()
debtrank_mx = mx_net.debtrank_multiplex(buffer_mx, a=0.2)
debtrank_mx_mean = mx_net.debtrank_multirank(buffer_mx, a=0.2)

cascade_mx, cascade_break_mx = mx_net.default_cascade_rank(thresholds,\
                                                           thresholds)
cascade_mx20, cascade_break_mx20 = mx_net.default_cascade_rank(thresholds,\
                                                           0.20)
cascade_mx33, cascade_break_mx33 = mx_net.default_cascade_rank(thresholds,\
                                                           0.333)
cascade_mx50, cascade_break_mx50 = mx_net.default_cascade_rank(thresholds,\
                                                           0.5)
cascade_mx66, cascade_break_mx66 = mx_net.default_cascade_rank(thresholds,\
                                                           0.66)
cascade_mx75, cascade_break_mx75 = mx_net.default_cascade_rank(thresholds,\
                                                           0.75)
cascade_mx80, cascade_break_mx80 = mx_net.default_cascade_rank(thresholds,\
                                                           0.8)
cascade_mx90, cascade_break_mx90 = mx_net.default_cascade_rank(thresholds,\
                                                           0.9)
cascade_mx33_10car, cascade_break_mx33_10car = mx_net.default_cascade_rank(1.1*thresholds,\
                                                           0.333)
cascade_mx33_20car, cascade_break_mx33_20car = mx_net.default_cascade_rank(1.2*thresholds,\
                                                           0.333)
cascade_mx33_33car, cascade_break_mx33_33car = mx_net.default_cascade_rank(1.33*thresholds,\
                                                           0.333)
cascade_mx33_0_90car, cascade_break_mx33_0_90car = mx_net.default_cascade_rank(0.9*thresholds,\
                                                           0.333)
cascade_mx33_0_80car, cascade_break_mx33_0_80car = mx_net.default_cascade_rank(0.8*thresholds,\
                                                           0.333)
cascade_mx33_0_67car, cascade_break_mx33_0_67car = mx_net.default_cascade_rank(0.67*thresholds,\
                                                           0.333)

# ranking network aggregate
pagerank_agg = agg_net.pagerank()
debtrank_agg = agg_net.debtrank_monoplex(buffer_agg, a=0.2)
katz_agg = agg_net.aggregate_katz(weights=thresholds)
cascade_agg, cascade_break_agg = agg_net.default_cascade_rank(thresholds, 0.0)

pagerank_equity = equity_net.pagerank()
debtrank_equity = equity_net.debtrank_monoplex(buffer_agg, a=0.2)
katz_equity = equity_net.aggregate_katz(weights=thresholds)
cascade_equity, cascade_break_equity = equity_net.default_cascade_rank(thresholds, 0.0)

pagerank_debt = debt_net.pagerank()
debtrank_debt = debt_net.debtrank_monoplex(buffer_agg, a=0.2)
katz_debt = debt_net.aggregate_katz(weights=thresholds)
cascade_debt, cascade_break_debt = debt_net.default_cascade_rank(thresholds, 0.0)

pagerank_bank = bank_net.pagerank()
debtrank_bank = bank_net.debtrank_monoplex(buffer_agg, a=0.2)
katz_bank = bank_net.aggregate_katz(weights=thresholds)
cascade_bank, cascade_break_bank = bank_net.default_cascade_rank(thresholds, 0.0)

# adding horizontal threshold heterogenity in monoplex networks
cascade_agg_10car, cascade_break_agg_10car = agg_net.default_cascade_rank(1.1*thresholds, 0.0)
cascade_agg_20car, cascade_break_agg_20car = agg_net.default_cascade_rank(1.2*thresholds, 0.0)
cascade_agg_33car, cascade_break_agg_33car = agg_net.default_cascade_rank(1.33*thresholds, 0.0)
cascade_agg_0_90car, cascade_break_agg_0_90car = agg_net.default_cascade_rank(0.9*thresholds, 0.0)
cascade_agg_0_80car, cascade_break_agg_0_80car = agg_net.default_cascade_rank(0.8*thresholds, 0.0)
cascade_agg_0_67car, cascade_break_agg_0_67car = agg_net.default_cascade_rank(0.67*thresholds, 0.0)
#cascade_equity, cascade_break_equity = equity_net.default_cascade_rank(thresholds, 0.0)
#cascade_debt, cascade_break_debt = debt_net.default_cascade_rank(thresholds, 0.0)
#cascade_bank, cascade_break_bank = bank_net.default_cascade_rank(thresholds, 0.0)

def print_top_n(rank, net, top_n=10, name=""):
    for i in range(top_n):
        country_index = sorted(rank, key=rank.__getitem__, reverse=True)[i]
        print(i, " ", name + ", ", net.nodes[country_index], ", ", rank[country_index] )

print(year)
#print_top_n(multirank_katz, mx_net, top_n=15, name="Katz mx")
#print_top_n(multirank_katz, mx_net, top_n=20, name="Katz mx")
#print_top_n(katz_agg, mx_net, top_n=20, name="Katz agg")
#print_top_n(cascade_break_mx, mx_net, top_n=20, name="Cascade mx")
#print_top_n(cascade_break_mx33, mx_net, top_n=20, name="Cascade mx 33")
#print_top_n(cascade_break_mx50, mx_net, top_n=20, name="Cascade mx 50")
#print_top_n(cascade_break_mx80, mx_net, top_n=20, name="Cascade mx 80")
#print_top_n(cascade_break_agg, agg_net, top_n=20, name="Cascade agg")
#print_top_n(debtrank_mx, mx_net, top_n=20, name="DebtRank mx")
#print_top_n(debtrank_mx_mean, mx_net, top_n=20, name="DebtRank mx")
#print_top_n(debtrank_agg, agg_net, top_n=20, name="Debtrank agg")


# saving aggregate centralities
cent_names_agg = ["PageRank","Katz", "DebtRank", "Cascade", \
                  "Cascade with tie break"]
centrality_agg = [pagerank_agg, katz_agg, debtrank_agg, cascade_agg, cascade_break_agg]
mc.save_nodecentralities(centrality_agg, cent_names_agg, agg_net, export_path_csv + \
                  exp_file_name_agg + str(year))

# saving monoplex centralities
cent_names_mono = ["PageRank equity","Katz equity", "DebtRank equity", "Cascade equity", \
                  "Cascade with tie break equity", "PageRank debt","Katz debt", "DebtRank debt", "Cascade", \
                    "Cascade with tie break debt", "PageRank bank","Katz bank", "DebtRank bank", "Cascade bank", \
                    "Cascade with tie break bank", "PageRank agg","Katz agg", "DebtRank agg", "Cascade agg", \
                    "Cascade with tie break agg",  "Cascade with tie break 10car", "Cascade with tie break 20car",\
                  "Cascade with tie break 33car", "Cascade with tie break 0_09car",\
                  "Cascade with tie break 0_08car", "Cascade with tie break 0_67car"]
centrality_mono = [pagerank_equity, katz_equity, debtrank_equity, cascade_equity, cascade_break_equity,\
                pagerank_debt, katz_debt, debtrank_debt, cascade_debt, cascade_break_debt,\
                pagerank_bank, katz_bank, debtrank_bank, cascade_bank, cascade_break_bank,\
                pagerank_agg, katz_agg, debtrank_agg, cascade_agg, cascade_break_agg, \
                cascade_break_agg_10car, cascade_break_agg_20car , cascade_break_agg_33car,\
                cascade_break_agg_0_90car, cascade_break_agg_0_80car, cascade_break_agg_0_67car]



def save_nodecentralities(centrality_list, centrality_name, network, csv_name):
    """ function that saves the centralities of nodes in a csv file.
    Output is csv file with ordered countries by rank and the score in a
    separate column.
    """
    f = open(csv_name + ".csv", "w")
    f.write("Rank")
    for c in centrality_name:
        f.write("," + c + "," + c +" score")
    f.write("\n")
    for i in range(network.nnodes):
        #write rank number
        f.write(str(i))
        for j,cent in enumerate(centrality_list):
            country_index = sorted(cent, key=cent.__getitem__, reverse=True)[i]
            country_nm = network.nodes[country_index]
            score = cent[country_index]
            # write country in rank i and score
            f.write("," + country_nm + "," + str(score))
        f.write("\n")
    f.close()

save_nodecentralities(centrality_mono, cent_names_mono, agg_net, export_path_csv + \
                  exp_file_name_mono + str(year))
#mc.save_nodecentralities(centrality_mono, cent_names_mono, agg_net, export_path_csv + \
#                  exp_file_name_mono + str(year))

# saving multiplex centralities
cent_names_mx = ["PageRank Multilayer", "Hubs", "Authorities", \
                 "MultiRank PageRank", "DebtRank", "DebtRank_layermean","Katz", \
                 "Cascade", "Cascade with tie break 20", "Cascade with tie break 33", "Cascade with tie break 50",\
                  "Cascade with tie break 66",  "Cascade with tie break 75",\
                  "Cascade with tie break 80",  "Cascade with tie break 90",\
                  "Cascade with tie break 33 10car", "Cascade with tie break 33 20car",\
                  "Cascade with tie break 33 33car", "Cascade with tie break 33 0_09car",\
                  "Cascade with tie break 33 0_08car", "Cascade with tie break 33 0_67car"]

centrality_mx = [pagerank_mx, hubs_mx, auth_mx, multirank_mx, debtrank_mx, \
                  debtrank_mx_mean,multirank_katz, cascade_mx, cascade_break_mx20,\
                  cascade_break_mx33,\
                  cascade_break_mx50, cascade_break_mx66,cascade_break_mx75,\
                  cascade_break_mx80, cascade_break_mx90, cascade_break_mx33_10car, \
                  cascade_break_mx33_20car, cascade_break_mx33_33car,\
                  cascade_break_mx33_0_90car, cascade_break_mx33_0_80car,\
                  cascade_break_mx33_0_67car]
mc.save_nodecentralities(centrality_mx, cent_names_mx, mx_net, export_path_csv + \
                  exp_file_name_mx + str(year))



# Other work

#cent_layer_names = ["MultiRank PageRank","Katz", "Money percent", "Katz_layerunw"]
#centrality_layer_mx = [multirank_layers, multirank_katz_layers, money_percent, \
#                        multirank_katz_layers_unw]
#mc.save_layercentralities(centrality_layer_mx , cent_layer_names, mx_net, export_path_csv + \
#                  exp_file_name_mxlayer + str(year))
########### Difference between using car distribution and only using mean car

#cascade_agg, cascade_break_agg = agg_net.default_cascade_rank(thresholds, 0.0)
#cascade_mx, cascade_break_mx = mx_net.default_cascade_rank(thresholds,\
#                                                           thresholds)
#cascade_agg_nocardist, cascade_break_agg_nocardist = agg_net.default_cascade_rank(thresholds.mean()*np.ones(agg_net.nnodes), 0.0)
#cascade_mx_nocardist, cascade_break_mx_nocardist = mx_net.default_cascade_rank(thresholds.mean()*np.ones(mx_net.nnodes),\
#                                                           thresholds.mean()*np.ones(mx_net.nnodes))
#
#cent_names_comparison = ["Agg No Car Cascade", "Agg Car Cascade ", "Mx No Car Cascade ", "Mx Car Cascade"]
#centrality_comparison = [cascade_break_agg_nocardist, cascade_break_agg, cascade_break_mx_nocardist, cascade_break_mx]
#
#save_centralities(centrality_comparison, cent_names_comparison, agg_net, export_path_csv + \
#                  "agg_vs_mx_vs_car_centrality" + str(year))
